@@ -55,6 +55,7 @@ typedef struct xsp_data {
 	char is_default;
 	char *alias;
 	char *filename;
+	char *umask_value;
 	char *run_xsp;
 	char *executable_path;
 	char *path;
@@ -164,6 +165,7 @@ add_xsp_server (apr_pool_t *pool, const char *alias, module_cfg *config, int is_
 	server->is_default = is_default;
 	server->alias = apr_pstrdup (pool, alias);
 	server->filename = NULL;
+	server->umask_value = NULL;
 	server->run_xsp = "True";
 	/* (Obsolete) server->executable_path = EXECUTABLE_PATH; */
 	server->path = NULL;
@@ -623,6 +625,7 @@ do_command (int command, apr_socket_t *sock, request_rec *r, int *result)
 	int32_t i;
 	uint32_t actual_size;
 	int status = 0;
+	apr_pool_t *temp_pool;
 
 	if (command < 0 || command >= LAST_COMMAND) {
 		ap_log_error (APLOG_MARK, APLOG_ERR, STATUS_AND_SERVER,
@@ -635,11 +638,14 @@ do_command (int command, apr_socket_t *sock, request_rec *r, int *result)
 	*result = OK;
 	switch (command) {
 	case SEND_FROM_MEMORY:
-		if (read_data_string (r->pool, sock, &str, &size) == NULL) {
+		apr_pool_create (&temp_pool, r->pool);
+		if (read_data_string (temp_pool, sock, &str, &size) == NULL) {
 			status = -1;
+			apr_pool_destroy (temp_pool);
 			break;
 		}
 		request_send_response_from_memory (r, str, size);
+		apr_pool_destroy (temp_pool);
 		break;
 	case GET_SERVER_VARIABLES:
 		ap_add_cgi_vars (r);
@@ -1095,7 +1101,18 @@ fork_mod_mono_server (apr_pool_t *pool, xsp_data *config)
 
 	setsid ();
 	chdir ("/");
-	umask (0077);
+	if (config->umask_value == NULL)
+		umask (0077);
+	else {
+		unsigned int uval;
+		DEBUG_PRINT (1, "unparsed umask value: %s", config->umask_value);
+		if (sscanf(config->umask_value, "%o", &uval) != 1) {
+			DEBUG_PRINT (1, "umask conversion to octal failed");
+			uval = 0077;
+		}
+		DEBUG_PRINT (1, "setting umask to %o", uval);
+		umask (uval);
+	}
 	DEBUG_PRINT (1, "child started");
 
 #ifdef DEBUG
@@ -1240,7 +1257,7 @@ write_string_to_buffer (char *buffer, int offset, const char *str)
 	buffer += offset;
 	tmp = (str != NULL) ? strlen (str) : 0;
 	le = LE_FROM_INT (tmp);
-	(*(int32_t *) buffer) = le;
+	memcpy (buffer, &le, sizeof (int32_t));
 	if (tmp > 0) {
 		buffer += sizeof (int32_t);
 		memcpy (buffer, str, tmp);
@@ -1311,7 +1328,7 @@ write_table_to_buffer (char *buffer, apr_table_t *table)
 	} while (t_elt < t_end);
 
 	count = LE_FROM_INT (count);
-	(*(int32_t *) buffer) = count;
+	memcpy (buffer, &count, sizeof (int32_t));
 	return (ptr - buffer);
 }
 
@@ -1365,12 +1382,12 @@ send_initial_data (request_rec *r, apr_socket_t *sock, char auto_app)
 	ptr += write_string_to_buffer (ptr, 0, r->connection->local_ip);
 	i = request_get_server_port (r);
 	i = LE_FROM_INT (i);
-	(*(int32_t *) ptr) = i;
+	memcpy (ptr, &i, sizeof (int32_t));
 	ptr += sizeof (int32_t);
 	ptr += write_string_to_buffer (ptr, 0, r->connection->remote_ip);
 	i = connection_get_remote_port (r->connection);
 	i = LE_FROM_INT (i);
-	(*(int32_t *) ptr) = i;
+	memcpy (ptr, &i, sizeof (int32_t));
 	ptr += sizeof (int32_t);
 	ptr += write_string_to_buffer (ptr, 0, connection_get_remote_name (r));
 	ptr += write_table_to_buffer (ptr, r->headers_in);
@@ -1790,6 +1807,10 @@ mono_register_hooks (apr_pool_t * p)
 #endif
 
 static const command_rec mono_cmds [] = {
+MAKE_CMD12 (MonoUnixUmask, umask_value,
+  "Value of the file mode creation mask (see umask(2))"
+  "Default: 0077"
+  ),
 MAKE_CMD12 (MonoUnixSocket, filename,
 	"Named pipe file name. Mutually exclusive with MonoListenPort. "
 	"Default: /tmp/mod_mono_server"
